"""
parser.http package (imdb package).

This package provides the IMDbHTTPMixIn class that should be mixed
with the imdb.IMDb class to access IMDb's data through the web interface;
the IMDb will automatically "mix-ins" itself with this class when
instantiated with the 'accessSystem' argument set to "http" or "web"
or "html" (this is the default).

Copyright 2004 Davide Alberani <davide.alberani@erlug.linux.it>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""

import urllib

from imdb.Movie import Movie
from imdb.Person import Person
from imdb._exceptions import IMDbDataAccessError, IMDbParserError
from movieParser import movie_parser, plot_parser
from searchMovieParser import search_movie_parser
from personParser import maindetails_Parser, filmography_Parser, bio_Parser
from searchPersonParser import search_person_parser


# Misc URLs
imdbURL_combdet = 'http://akas.imdb.com/title/tt%s/combined'
imdbURL_plot = 'http://akas.imdb.com/title/tt%s/plotsummary'

imdbURL_person_main = 'http://akas.imdb.com/name/nm%s/maindetails'
imdbURL_person_bio = 'http://akas.imdb.com/name/nm%s/bio'
imdbURL_person_movies = 'http://akas.imdb.com/name/nm%s/filmoyear'

imdbURL_search = 'http://akas.imdb.com/find?%s'


class IMDbURLopener(urllib.FancyURLopener):
    """Fetch web pages and handle errors."""
    # XXX: does it make sense to override the http_error_default() method,
    #      or it's better to override only single methods, like
    #      http_error_404()?
    def http_error_default(self, url, fp, errcode, errmsg, headers):
        raise IMDbDataAccessError, {'url': 'http:%s' % url,
                                    'errcode': errcode,
                                    'errmsg': errmsg,
                                    'headers': headers}

    def open_unknown(self, fullurl, data=None):
        raise IMDbDataAccessError, {'fullurl': fullurl,
                                    'data': str(data)}

    def open_unknown_proxy(self, proxy, fullurl, data=None):
        raise IMDbDataAccessError, {'proxy': str(proxy),
                                    'fullurl': fullurl,
                                    'data': str(data)}


class IMDbHTTPMixIn:
    """The Mix-ins class used to access IMDb's data through the web."""
    # The object used to access HTTP pages.
    urlOpener = IMDbURLopener()
    # XXX: IMDb's web server doesn't like crawlers, so let's fake
    #      to be Mozilla.
    #      Wow!  I'm shocked by my total lack of ethic! <g>
    urlOpener.addheaders = [('User-agent', 'Mozilla/5.0')]
    # The URL for the "combined details" and "plot summary" pages.

    def get_imdbMovieID(self, movieID):
        """Translate a movieID in an imdbID; in this implementation
        the movieID _is_ the imdbID.
        """
        return movieID

    def get_imdbPersonID(self, personID):
        """Translate a personID in an imdbID; in this implementation
        the personID _is_ the imdbID.
        """
        return personID

    def set_proxy(self, proxy):
        """Set the web proxy to use.
        
        It should be a string like 'http://localhost:8080/'; if the
        string is empty, no proxy will be used.
        If set, the value of the environment variable HTTP_PROXY is used."""
        if not proxy:
            if self.urlOpener.proxies.has_key('http'):
                del self.urlOpener.proxies['http']
        else:
            if not proxy.lower().startswith('http://'):
                proxy = 'http://%s' % proxy
            self.urlOpener.proxies.update({'http': proxy})

    def get_proxy(self):
        """Return the used proxy or an empty string."""
        return self.urlOpener.proxies.get('http', '')

    def search_movie(self, title, results=20):
        """Search for a given title; return a list of Movie objects."""
        try:
            int(results)
        except (ValueError, OverflowError):
            results = 20
        # The URL of the query.
        params = urllib.urlencode({'tt': 'on', 'mx': str(results), 'q': title})
        url = imdbURL_search % params
        try:
            query_url = self.urlOpener.open(url)
            search_result = query_url.read()
            query_url.close()
            self.urlOpener.close()
        except IOError, e:
            raise IMDbDataAccessError, {'errcode': e.errno,
                                        'errmsg': str(e.strerror),
                                        'url': url}
        res = search_movie_parser.parse(search_result)
        return [Movie(movieID=mi, movieData=md, accessSystem='http')
                for mi, md in res][:results]

    def _get_movie_data(self, movieID):
        """Return a dictionary suitable for the Movie class, from the
        given movieID.
        """
        # URL of the "combined details" and "plot summary" pages.
        url_combdet = imdbURL_combdet % movieID
        url_plot = imdbURL_plot % movieID
        try:
            cdu = self.urlOpener.open(url_combdet)
            md = cdu.read()
            cdu.close()
            self.urlOpener.close()
        except IOError, e:
            # XXX: do I need to include a finally clause to clean
            #      the state of the used objects?
            raise IMDbDataAccessError, {'errcode': e.errno,
                                        'errmsg': str(e.strerror),
                                        'url': url_combdet}
        pd = ''
        try:
            plu = self.urlOpener.open(url_plot)
            pd = plu.read()
            plu.close()
            self.urlOpener.close()
        except IOError, e:
            # XXX: I'm not sure: we can return a lot of information
            #      about a movie even if we've not got the plot, but after
            #      all a serious trouble occurred (the IMDb web server show
            #      a web page even for movie without the "plot summary"
            #      section).
            pass
        # Parse "main details" and "plot summary" pages.
        movie_data = movie_parser.parse(md)
        plot_data = {}
        if pd:
            try:
                plot_data = plot_parser.parse(pd)
            except IMDbParserError:
                # XXX: see above...
                pass
        # Merge the two dictionaries returned by the parsers.
        movie_data.update(plot_data)
        return movie_data

    def search_person(self, name, results=20):
        """Search for a given name; return a list of Person objects."""
        try:
            int(results)
        except (ValueError, OverflowError):
            results = 20
        # The URL of the query.
        params = urllib.urlencode({'nm': 'on', 'mx': str(results), 'q': name})
        url = imdbURL_search % params
        try:
            query_url = self.urlOpener.open(url)
            search_result = query_url.read()
            query_url.close()
            self.urlOpener.close()
        except IOError, e:
            raise IMDbDataAccessError, {'errcode': e.errno,
                                        'errmsg': str(e.strerror),
                                        'url': url}
        res = search_person_parser.parse(search_result)
        return [Person(personID=mi, personData=md, accessSystem='http')
                for mi, md in res][:results]

    def _get_person_data(self, personID):
        """Return a dictionary suitable for the Person class, from the
        given personID.
        """
        mainURL = imdbURL_person_main % personID
        bioURL = imdbURL_person_bio % personID
        moviesURL = imdbURL_person_movies % personID
        try:
            pu = self.urlOpener.open(mainURL)
            pd = pu.read()
            pu.close()
            self.urlOpener.close()
        except IOError, e:
            # XXX: do I need to include a finally clause to clean
            #      the state of the used objects?
            raise IMDbDataAccessError, {'errcode': e.errno,
                                        'errmsg': str(e.strerror),
                                        'url': mainURL}
        try:
            bu = self.urlOpener.open(bioURL)
            bd = bu.read()
            bu.close()
            self.urlOpener.close()
        except IOError:
            # XXX: ignore errors retrieving the biography.
            pass
        try:
            mu = self.urlOpener.open(moviesURL)
            md = mu.read()
            mu.close()
            self.urlOpener.close()
        except IOError:
            # XXX: ignore errors retrieving the filmography.
            #      It's an important information... is it right to
            #      ignore it?
            pass
        data = {}
        person_det = maindetails_Parser.parse(pd)
        bio_det = {}
        if bd:
            try:
                bio_det = bio_Parser.parse(bd)
            except IMDbParserError:
                pass
        movie_det = {}
        if md:
            try:
                movie_det = filmography_Parser.parse(md)
            except IMDbParserError:
                pass
        # Merge the three dictionaries returned by the parsers.
        data.update(person_det)
        data.update(bio_det)
        data.update(movie_det)
        return data


