"""
parser.http.searchPersonParser module (imdb package).

This module provides the HTMLSearchPersonParser class (and the
search_person_parser instance), used to parse the results of a search
for a given person.
E.g., when searching for the name "Mel Gibson", the parsed page would be:
    http://akas.imdb.com/find?q=Mel+Gibson&nm=on&mx=20

Copyright 2004 Davide Alberani <davide.alberani@erlug.linux.it>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""

from htmllib import HTMLParser
from formatter import NullFormatter

from imdb.utils import analyze_name

from imdb._exceptions import IMDbParserError
from utils import get_attr_value, re_imdbID


# XXX: this class inherits from HTMLParser; see the documentation for
#      the "htmllib" and "sgmllib" modules.
class BasicPersonParser(HTMLParser):
    """Simply get the name of a person and the imdbID.
    
    It's used by the HTMLSearchPersonParser class to return a result
    for a direct match (when a search on IMDb results in a single
    person, the web server sends directly the person page.
    """
    def __init__(self, formatter=NullFormatter(), verbose=0):
        HTMLParser.__init__(self, formatter, verbose)
        self.entitydefs['nbsp'] = ' '

    def reset(self):
        """Reset the parser."""
        HTMLParser.reset(self)
        self.__in_title = 0
        self.__name = ''
        self.__current_personID = ''
        self.__personID = ''

    def get_result(self):
        """Return a list with a single tuple (imdb, {title_dict})."""
        res = {}
        pid = self.__personID
        if pid:
            rpid = re_imdbID.findall(pid)
            if rpid:
                pid = rpid[-1]
        if self.__personID and self.__name:
            d = analyze_name(self.__name)
            res.update(d)
        return [(pid, res)]

    def start_title(self, attrs):
        self.__in_title = 1

    def end_title(self):
        self.__in_title = 0

    def start_a(self, attrs):
        href = get_attr_value(attrs, 'href')
        if href and href.lower().startswith('/name'):
            self.__current_personID = href
    
    def end_a(self):
        pass
    
    def do_img(self, attrs):
        # XXX: here we're searching for the "pageflicker" links; the
        #      one around the "main details" image (the square/stop icon)
        #      is the only reliable source to read the imdbID from...
        alt = get_attr_value(attrs, 'alt')
        if alt and alt.strip().lower() == 'main details' \
                and self.__current_personID:
            self.__personID = self.__current_personID

    def handle_data(self, data):
        if self.__in_title:
            self.__name += data

    def anchor_bgn(self, href, name, type):
        pass

    def anchor_end(self):
        pass
    
    def handle_image(self, src, alt, *args):
        pass

    def error(self, message):
        raise IMDbParserError, 'HTML parser error: "%s"' % str(message)

    def parse(self, html_res_page):
        """Parse the given html page and return a list of ('imdbID',
        {name_dict}) tuples.
        """
        self.reset()
        self.feed(html_res_page)
        return self.get_result()


# XXX: this class inherits from HTMLParser; see the documentation for
#      the "htmllib" and "sgmllib" modules.
class HTMLSearchPersonParser(HTMLParser):
    """Parse the html page that the IMDb web server shows when the
    "new search system" is used.
    """
    def __init__(self, formatter=NullFormatter(), verbose=0):
        HTMLParser.__init__(self, formatter, verbose)
        self.entitydefs['nbsp'] = ' '

    def reset(self):
        """Reset the parser."""
        HTMLParser.reset(self)
        self.__begin_list = 0
        self.__results = []
        self.__in_title = 0
        self.__in_list = 0
        self.__current_imdbID = ''
        self.__is_name = 0
        self.__name = ''
        self.__no_more = 0

    def get_results(self):
        """Return a list of tuples (imdbID, {name_dict})."""
        return self.__results

    def start_title(self, attrs):
        self.__in_title = 1

    def end_title(self):
        self.__in_title = 0

    def start_tr(self, attrs):
        self.__in_list = 1

    def end_tr(self):
        self.__in_list = 0

    def start_a(self, attrs):
        href = get_attr_value(attrs, 'href')
        if href and href.lower().startswith('/name'):
            nr = re_imdbID.findall(href[6:])
            if not nr:
                return
            self.__current_imdbID = nr[0]
            self.__is_name = 1

    def end_a(self):
        pass

    def start_small(self, attrs):
        self.__no_more = 1

    def end_small(self):
        pass

    def start_td(self, attrs):
        pass
    
    def end_td(self):
        if self.__in_list and self.__is_name and self.__current_imdbID \
                and self.__name:
            res = {}
            d = analyze_name(self.__name.strip())
            res.update(d)
            self.__results.append((self.__current_imdbID.strip(), d))
            self.__name = ''
            self.__current_imdbID = ''
            self.__is_name = 0
            self.__in_name = 0
        self.__no_more = 0

    def handle_data(self, data):
        sldata = data.strip().lower()
        if self.__in_title:
            if data.strip().lower() != 'imdb name search':
                rawdata = self.rawdata
                self.reset()
                bpp = BasicPersonParser()
                self.__results = bpp.parse(rawdata)
        elif self.__in_list and self.__is_name and not self.__no_more:
            self.__name += data
        elif sldata.startswith('approximate match') or \
                sldata.startswith('exact match'):
            self.__begin_list = 1
    
    def anchor_bgn(self, href, name, type):
        pass

    def anchor_end(self):
        pass
    
    def handle_image(self, src, alt, *args):
        pass

    def error(self, message):
        raise IMDbParserError, 'HTML parser error: "%s"' % str(message)

    def parse(self, html_res_page):
        """Parse the given html page (as built by the "new search system")
        and return a list of ('imdbID', {name_dict}) tuples.
        """
        self.reset()
        self.feed(html_res_page)
        return self.get_results()


search_person_parser = HTMLSearchPersonParser()


